# Copyright (C) 2003-2004 LAAS/CNRS 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
#    - Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# $LAAS$
#
# iRobot B21r model, with LAAS equipement
# Aurelie Clodic, Matthieu Herrb
#

package provide Gdhe::models 1.0

# Global parameters
namespace eval b21r {
    set r .65
    set h .45
    set w [ expr 3.1416 * $b21r::r / 24 ]
}

objectDelete RackhamPlatineInf
objectDelete RackhamPlatineMid
objectDelete RackhamPlatineSup
objectDelete B21r
objectDelete Rackham

#----------------------------------------------------------------------
### 
### platine
###
proc b21rplatine {azi site} {
    pushMatrix
    object RackhamPlatineInf {
        color 30 30 30
        box -0.02 0.0 0 0.08 0.05 0.05
        color 180 180 180
        cylinder -0.075 0 0.025 x 0.012 0.02
    }
    rotate $azi 0 0 1
    object RackhamPlatineMid {
        color 30 30 30
        box -0.02 0 0.051 0.08 0.05 0.05
        color 180 180 180
        cylinder -0.075 0 0.075 x 0.012 0.02
        translate 0 0 0.075
    }
    rotate $site 0 -1 0
    object RackhamPlatineSup {
        color 30 30 30
        box 0 -0.028 -0.01 0.02 0.005 0.05
        box 0  0.028 -0.01 0.02 0.005 0.05
        color 100 100 100
        box 0 0 0.04 0.02 0.06 0.005
	# Camera 
	color 100 120 120
	box 0 0 0.045 0.025 0.06 0.05
	cylinder 0.0125 0 0.065 x 0.028 0.025
    }
    popMatrix
}

# upper US sensor
proc b21rus {} {
    pushMatrix
    translate [expr $b21r::r/2.0] 0 0 
    rotate 90 0 0 1
    rotate 90 1 0 0 
    color  200 200 80
    cylinder 0 .0 0.0 z .05 .002
    popMatrix
}

# lower US sensor
proc b21rus2 {} {
    pushMatrix
    translate [expr $b21r::r/2.0 + 0.01] 0 0
    rotate 90 0 0 1
    rotate 90 1 0 0 
    color 180 180 60
    box 0 0 0 .01 .02 .002
    popMatrix
}

# The whole robot
proc b21r {} {
    global b21r::r b21r::w b21r::h

    # wheels
    object B21rWheels {
        color 30 30 30
        cylinder 0.23 -0.015 0.05 y 0.1 0.03
        cylinder -0.23 -0.015 0.05 y 0.1 0.03
        cylinder -0.01 0.215 0.05 y 0.1 0.03
        cylinder -0.01 -0.245 0.05 y 0.1 0.03
    }

    # main body
    object B21r { 
	color 20 20 20 
	cylinder 0 0 .09 z $b21r::r $b21r::r .01 24
	color 180 0 0
	cylinder 0 0 .1 z [expr $b21r::r + 0.02] [expr $b21r::r + 0.02] .32 24
	color 20 20 20 
	cylinder 0 0 .42 z [expr $b21r::r -0.02] [expr $b21r::r - 0.02] .07 24
	color 180 0 0 
	cylinder 0 0 .49 z $b21r::r $b21r::r .6 24
	color 20 20 20 
	cylinder 0 0 1.09 z $b21r::r $b21r::r .01 24

	for { set i 0 } { $i < 24 } { incr i } {
	    pushMatrix
	    rotate [expr 360/24 * $i ] 0 0 1
	    translate 0 0 .3
	    b21rus2
	    translate 0 0 .5
	    b21rus
	    popMatrix
	}

	# Command box
	pushMatrix
	color 180 0 0 
	set c1 { prism 19 0 0 .07 }
	set c2 { prism 19 0 0 .005 }
	for { set i -7 } { $i <= 7 } { incr i } { 
	    lappend c1 [ expr cos(6.28/24 * $i) * $b21r::r / 2.1]
	    lappend c1 [ expr sin(6.28/24 * $i) * $b21r::r / 2.1]
	    lappend c1 0
	    lappend c2 [ expr cos(6.28/24 * $i) * $b21r::r / 2.1]
	    lappend c2 [ expr sin(6.28/24 * $i) * $b21r::r / 2.1]
	    lappend c2 0
	}
	lappend c1 -0.084 0.2 0
	lappend c1 0.05 0.2 0 
	lappend c1 0.05 -0.2 0
	lappend c1 -0.084 -0.2 0

	lappend c2 -0.084 0.2 0
	lappend c2 0.05 0.2 0
	lappend c2 0.05 -0.2 0
	lappend c2 -0.084 -0.2 0


	translate 0 0 1.2
	eval $c1
	translate 0 0 .07
	color 20 20 20
	eval $c2
	popMatrix 

	pushMatrix
	translate 0 -.3 1.1
	color 60 60 60
	prism 4 0 .01 0 \
	    0.04 0 0 \
	    -0.04 0 0 \
	    0 0 .1 \
	    0.08  0 .1
	translate 0 .58 0
	color 60 60 60
	prism 4 0 .01 0 \
	    0.04 0 0 \
	    -0.04 0 0 \
	    0 0 .1 \
	    0.08  0 .1
	popMatrix
	

    }
}

#
# The B21r, with all LAAS equipment 
#
proc rackham {} {
    b21r
    object Rackham {
	# Touch screen 
	pushMatrix
	translate -0.17 0 1.1
	color 60 60 60 
	box 0 0 0  .07 .4 .082
	translate 0 0 .07
	rotate 30 0 1 0 
	box 0 0 0 .07 .4 .3 
	color 180 180 200
	box -0.033 0.0 0.04 .01 .3 .23
	popMatrix

	# colone
	pushMatrix
	color 80 80 80
	translate 0.1 0 1
	cylinder 0 0 0 z .05 .8
	pushMatrix
	translate -.05 0 .6
	cylinder 0 0 0 x .12 .12 .03 24
	popMatrix
	translate 0 0 .8
	cylinder 0 0 0 z .5 .5 .04 24
	popMatrix
	
	# rear Camera 
	color 100 100 100
	pushMatrix
	translate 0.1 -0.12 1.80
	rotate 90 -1 0 0 
	prism 4 0 0 0.015 \
	    -0.02 0 0 \
	    0.02 0 0 \
	    0.01 0.04 0 \
	    -0.01 0.04 0
	prism 4 0 0 0.015 \
	    -0.02 0 -0.08 \
	    0.02 0 -0.08 \
	    0.01 0.04 -0.08 \
	    -0.01 0.04 -0.08
	color 100 120 120
	translate 0 0.03 0
	rotate -20 0 0 1
	# repere 0.2
	box  -0.02 0 -0.06 0.08 0.05 0.06
	cylinder -0.08 0 -0.03 x 0.03 0.03
	popMatrix

	# Helmet
	pushMatrix
	translate 0.1 0 1.84
	helmet 0.25
	popMatrix
    }
    translate 0.2 0 1.84
    b21rplatine 0 -30 
    
}

proc helmet {r} {
    
    set pi 3.1415926535
    set di [ expr $pi / 20 ]
    set dj [ expr $pi / 20 ]

    color 80 80 80 
    cullFace f
    concave f

    for { set i [expr $pi/3] } { $i < 5*$pi/3 } { set i [expr $i + $di] } {
	for { set j 0 } { $j < $pi / 2.0 } { set  j [expr $j + $dj] } {
	    set ci [ expr cos($i) ]
	    set si [ expr sin($i) ]
	    set cj [ expr cos($j) ]
	    set sj [ expr sin($j) ]
	    set cii [ expr cos ( $i + $di ) ]
	    set sii [ expr sin ( $i + $di ) ]
	    set cjj [ expr cos ( $j + $dj ) ]
	    set sjj [ expr sin ( $j + $dj ) ]

	    set x1 [ expr $r * $ci * $cj ]
	    set y1 [ expr $r * $si * $cj ]
	    set z1 [ expr $r * $sj ]

	    set x2 [ expr $r * $cii * $cj ]
	    set y2 [ expr $r * $sii * $cj ]
	    set z2 [ expr $r * $sj ]

	    set x3 [ expr $r * $cii * $cjj ]
	    set y3 [ expr $r * $sii * $cjj ]
	    set z3 [ expr $r * $sjj ]

	    set x4 [ expr $r * $ci * $cjj ]
	    set y4 [ expr $r * $si * $cjj ]
	    set z4 [ expr $r * $sjj ]

	    #puts "$x1 $y1 $z1"
	    #puts "$x2 $y2 $z2" 
	    #puts "$x3 $y3 $z3"
	    #puts "$x4 $y4 $z4"
	    #puts ""
	    polygon 4 $x1 $y1 $z1 $x2 $y2 $z2 $x3 $y3 $z3 $x4 $y4 $z4
	}
    }
    color 255 40 0 
    set di [ expr $pi / 12.0 ]
    for { set i 0.0 } { $i < 2 * $pi } { set i [ expr $i + $di ] } {
	set ci [ expr cos($i) ]
	set si [ expr sin($i) ]
	set x [ expr $r * cos($i) ]
	set y [ expr $r * sin($i) ]
	sphere $x $y -0.01 0.005
    }
    for { set i 1 } { $i <= 6 } { incr i } {
	set ci [ expr cos($i*$pi/12.0) ]
	set si [ expr sin($i*$pi/12.0) ]
	set x [ expr -$r * $ci ]
	set z [ expr $r * $si ]
	sphere $x 0 $z 0.005
    }
}
